/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include "project.h"
#include <stdlib.h>
#include <stdio.h>

#include "diskio.h"
#include "ff.h"

char uartInputBuffer[255];
uint8 uartIBufferPtr = 0;

void SendMenu();

void Test_A();
void Test_B();

void FatFsError(FRESULT result);

int main(void)
{
    __enable_irq(); /* Enable global interrupts. */
    
    char uartchar;
    
    /* Place your initialization/startup code here (e.g. MyInst_Start()) */
    UART_Start();
    setvbuf(stdin,NULL,_IONBF,0); //turn off buffer of UART

    SendMenu();
    
    /* Start SPI bus. */
    SPI_Start();
    Cy_GPIO_Write(SPI_CS_PORT, SPI_CS_NUM, 1);
    
    printf("Entering for loop\n\r");
    
    for(;;)
    {
        printf("Waiting for user input\n\r");
        uartchar = getchar();
        printf("Got input\n\r");
        
        if (uartchar != 0)
        {

            printf("%c\n\r",uartchar);
            
            switch (uartchar)
            {
                case 'a':
                case 'A':
                {
                    Test_A(); 
                    printf("\nSelect test :>"); break;
                }; break;
                
                case 'b':
                case 'B':
                {
                    Test_B(); 
                    printf("\nSelect test :>"); break;
                }; break;
                    
                case '?': 
                    SendMenu(); break;
                    
                case 'Y':
                    {}; break;
                    
                case 'Z':
                    {}; break;
                    
                default:
                    printf("\nSelect test :>"); break;
            }
        }

    }
}

void SendMenu()
{
    printf("\n\nPSoC FatFs. PSoC 6 FatFs Test/Demo.\n\r");
    printf("Hacking Chips. 12/2015.\n\n\r");
    
    printf("\t(A) - Show 'root' directory listing.\n\r");
    printf("\t(B) - Create directories, create files.\n\r");    
    
    printf("\t(?) - Show menu.\n\r");
    
    printf("Select test :>\n\r");    
}

void Test_A()
{
    FATFS fatFs;
    FIL fileO;
    DIR directoryO;
    FILINFO fileInfo;
    uint8 resultF;
    FATFS *fatFsPtr;
    
    char str[100];
    uint32 freeclusters, freesectors, totalsectors;

    /* Mount sdcard. */
    resultF = f_mount(&fatFs, "", 1);
    
    printf("resultF %d\n\r",resultF);
    
    if (resultF == RES_OK)
    {
        /* read volume label. */
        resultF = f_getlabel("", str, 0);
        
        if (resultF != FR_OK)
        {
            FatFsError(resultF);
            return;
        }
        
        printf("\nVolume label: %s \n\n",str);
        
        /* Read directory contents. */
        resultF = f_opendir(&directoryO, "/");      // open 'root' directory.
        if (resultF != 0)
        {
            FatFsError(resultF);
            return;
        }
        
        while(1)
        {
            resultF = f_readdir(&directoryO, &fileInfo);
            if (resultF != 0)
            {
                FatFsError(resultF);
                break;
            }
            
            if (fileInfo.fname[0] == 0)             // end of directory?
            {
                break;     
            }
            
            if (fileInfo.fattrib & AM_DIR)          // is it a directory?
            {
                printf(fileInfo.fname);
                printf("\t\t\t<DIR>\n");
            }
            else                                    // it is a file.
            {
                printf(fileInfo.fname);
                printf("\t\t\t");     
                
                itoa(fileInfo.fsize, str, 10);
                printf(str);
                putchar('\n');
            }
                        
        }
        
        /* get volume bytes info. */
        resultF = f_getfree("", &freeclusters, &fatFsPtr);
        
        if (resultF != FR_OK)
        {
            FatFsError(resultF);
            return;
        }

        totalsectors = (fatFsPtr->n_fatent - 2) * fatFsPtr->csize;
        freesectors = freeclusters * fatFsPtr->csize;  

        itoa(totalsectors, str, 10);
        printf("\n\tTotal sectors: %s\n",str);
        itoa(freesectors, str, 10);
        printf("\tFree sectors: %s\n",str); 
        itoa((totalsectors /2), str, 10);
        printf("\tDrive space: %s\n",str); 
        itoa((freesectors /2), str, 10);
        printf("\tFree space: %s\n",str);        
        
        f_closedir(&directoryO);
        
        /* unmount */
        resultF = f_mount(NULL, "", 1);
        if (resultF != FR_OK)
        {
            FatFsError(resultF);
            return;
        }
    }
    else FatFsError(resultF);
}

void Test_B()
{
    FATFS fatFs;
    FIL fileO;
    DIR directoryO;
    FILINFO fileInfo;
    uint8 resultF;
    
    char str[100];
    uint32 freeclusters, freesectors, totalsectors;

    /* Mount sdcard. */
    resultF = f_mount(&fatFs, "", 1);
    
    if (resultF == RES_OK)
    {

        /* Create directory 'testb1' inside 'root' directory. */
        printf("\n...Creating directory 'testb1' inside root directory");
        resultF = f_mkdir("/testb1");
        
        if (resultF == RES_OK)
        {
            printf("\n...Creation of 'testb1' OK...");
        }
        else
        {
            if (resultF == FR_EXIST)
            {
                printf("\n...Directory 'testb1' exists... DELETING...");
                resultF = f_unlink("/testb1");
                if (resultF != RES_OK) FatFsError(resultF);
                printf("\n...Creating directory 'testb1' inside root directory");
                resultF = f_mkdir("/testb1");
                if (resultF != RES_OK) FatFsError(resultF);
            }
            else
            {
                FatFsError(resultF);
                return;
            }
        }
        
        /* Create directory 'testb2' inside 'root' directory. */
        printf("\n...Creating directory 'testb2' inside root directory");
        resultF = f_mkdir("/testb2");
        
        if (resultF == RES_OK)
        {
            printf("\n...Creation of 'testb2' OK...");
        }
        else
        {
            if (resultF == FR_EXIST)
            {
                printf("\n...Directory 'testb2' exists... DELETING...");
                resultF = f_unlink("/testb2");
                if (resultF != RES_OK) FatFsError(resultF);
                printf("\n...Creating directory 'testb2' inside root directory");
                resultF = f_mkdir("/testb2");
                if (resultF != RES_OK) FatFsError(resultF);
            }
            else
            {
                FatFsError(resultF);
                return;
            }
        }
        
        /* Create file inside 'testb2' */
        printf("\n...CD into 'testb2' directory.");
        resultF = f_chdir("/testb2");
        if (resultF != RES_OK) 
        {
            FatFsError(resultF);
            return;
        }
        
        printf("\n...Creating file 'testtxt.txt' inside 'testb2' directory.");
        resultF = f_open(&fileO, "testtxt.txt", FA_WRITE | FA_CREATE_NEW);
        
        if (resultF == FR_OK)
        {
            printf("...writing\n");
            
            resultF = f_puts("write line 1\r\n", &fileO);
            resultF = f_puts("write line 2\r\n", &fileO);
            resultF = f_puts("write line 3\r\n", &fileO);
            
    		resultF = f_close(&fileO);								/* Close the file */
            
            printf("...end writing\n");

    	} 
        else 
        {
            if (resultF == FR_EXIST)
            {
                printf("\n...File exists... DELETING...");
                resultF = f_unlink("testtxt.txt");
                if (resultF != RES_OK) FatFsError(resultF);
                printf("\n...Repeat test...");
            }
            else
            {
                FatFsError(resultF);
                return;
            }
        }
    }
    else FatFsError(resultF);    
}

void FatFsError(FRESULT result)
{
    switch (result)
    {
        case FR_DISK_ERR:
            printf("\n    error: (FR_DISK_ERR) low level error.\n"); break;
            
        case FR_INT_ERR:
            printf("\n    error: (FR_INT_ERR)\n"); break; 
            
        case FR_NOT_READY:
            printf("\n    error: (FR_NOT_READY) sdcard not ready.\n"); break;
            
        case FR_NO_FILE:
            printf("\n    error: (FR_NO_FILE) invalid file.\n"); break;
            
        case FR_NO_PATH:
            printf("\n    error: (FR_NO_PATH) invalid path.\n"); break;
            
        case FR_INVALID_NAME:
            printf("\n    error: (FR_INVALID_NAME) invalid name.\n"); break;
            
        case FR_DENIED:
            printf("\n    error: (FR_DENIED) operation denied.\n"); break;
            
        case FR_EXIST:
            printf("\n    error: (FR_EXIST) it exists yet...\n"); break;
            
        case FR_INVALID_OBJECT:
            printf("\n    error: (FR_INVALID_OBJECT)\n"); break;
            
        case FR_WRITE_PROTECTED:
            printf("\n    error: (FR_WRITE_PROTECTED)\n"); break;
            
        case FR_INVALID_DRIVE:
            printf("\n    error: (FR_INVALID_DRIVE)\n"); break;
            
        case FR_NOT_ENABLED:
            printf("\n    error: (FR_NOT_ENABLED) sdcard unmounted.\n"); break;
            
        case FR_NO_FILESYSTEM:
            printf("\n    error: (FR_NO_FILESYSTEM) no valid FAT volume.\n"); break;  
            
        case FR_MKFS_ABORTED:
            printf("\n    error: (FR_MKFS_ABORTED)\n"); break;
            
        case FR_TIMEOUT:
            printf("\n    error: (FR_TIMEOUT)\n"); break;
            
        case FR_LOCKED:
            printf("\n    error: (FR_LOCKED)\n"); break;
            
        case FR_NOT_ENOUGH_CORE:
            printf("\n    error: (FR_NOT_ENOUGH_CORE)\n"); break;     
            
        case FR_TOO_MANY_OPEN_FILES:
            printf("\n    error: (FR_TOO_MANY_OPEN_FILES)\n"); break;
            
        case FR_INVALID_PARAMETER:
            printf("\n    error: (FR_INVALID_PARAMETER)\n"); break; 
            
        default: {}; break;
    }
}

/* [] END OF FILE */
